<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2021 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class Tarkpay extends PaymentModule
{
    protected $config_form = false;

    public $api_key;
    public $client_id;

    public function __construct()
    {
        $this->name = 'tarkpay';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->author = 'Codecloud OÜ';
        $this->need_instance = 1;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Tarkpay');
        $this->description = $this->l('Accept payments by bank link');

        $this->confirmUninstall = $this->l('Are you sure you wan\'t to uninstall Tarkpay?');

        $this->limited_countries = array('EE', 'LV', 'LT');

        $this->limited_currencies = array('EUR');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);

        $config = Configuration::getMultiple(array('TARKPAY_API_KEY', 'TARKPAY_CLIENT_ID', 'TARKPAY_TITLE', 'TARKPAY_LIVE_MODE'));
        if (!empty($config['TARKPAY_CLIENT_ID'])) {
            $this->client_id = $config['TARKPAY_CLIENT_ID'];
        }
        if (!empty($config['TARKPAY_API_KEY'])) {
            $this->api_key = $config['TARKPAY_API_KEY'];
        }
        if (!empty($config['TARKPAY_TITLE'])) {
            $this->title = $config['TARKPAY_TITLE'];
        }
        if ($config['TARKPAY_LIVE_MODE']) {
            $this->url = "https://tarkpay.com/api/v1";
        } else {
            $this->url = "https://sandbox.tarkpay.com/api/v1";
        }
        if (!isset($this->client_id) || !isset($this->api_key)) {
            $this->warning = $this->l('Details must be configured before using this module.');
        }
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false)
        {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        if (in_array($iso_code, $this->limited_countries) == false)
        {
            $this->_errors[] = $this->l('This module is not available in your country');
            return false;
        }

        // Registration order status
        if (!$this->installOrderState()) {
            $this->_errors[] = $this->l('Status install failed.');
            return false;
        }
        Configuration::updateValue('TARKPAY_LIVE_MODE', false);
        Configuration::updateValue('TARKPAY_CLIENT_ID', null);
        Configuration::updateValue('TARKPAY_API_KEY', null);
        Configuration::updateValue('TARKPAY_TITLE', null);

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('payment') &&
            $this->registerHook('paymentReturn') &&
            $this->registerHook('paymentOptions');
    }

    public function uninstall()
    {
        Configuration::deleteByName('TARKPAY_LIVE_MODE');
        Configuration::deleteByName('TARKPAY_CLIENT_ID');
        Configuration::deleteByName('TARKPAY_API_KEY');
        Configuration::deleteByName('TARKPAY_TITLE');

        return parent::uninstall();
    }
/**
     * Create order state
     * @return boolean
     */
    public function installOrderState()
    {
        if (!Configuration::get('TARKPAY_OS_WAITING')
            || !Validate::isLoadedObject(new OrderState(Configuration::get('TARKPAY_OS_WAITING')))) {
            $order_state = new OrderState();
            $order_state->name = array();
            foreach (Language::getLanguages() as $language) {
                if (Tools::strtolower($language['iso_code']) == 'fr') {
                    $order_state->name[$language['id_lang']] = 'En attente de paiement Tarkpay';
                } else {
                    $order_state->name[$language['id_lang']] = 'Awaiting for Tarkpay payment';
                }
            }
            $order_state->send_email = false;
            $order_state->color = '#4169E1';
            $order_state->hidden = false;
            $order_state->delivery = false;
            $order_state->logable = false;
            $order_state->invoice = false;
            $order_state->module_name = $this->name;
            if ($order_state->add()) {
                $source = _PS_MODULE_DIR_ . 'tarkpay/logo.png';
                $destination = _PS_ROOT_DIR_ . '/img/os/' . (int)$order_state->id . '.gif';
                copy($source, $destination);
            }

            if (Shop::isFeatureActive()) {
                $shops = Shop::getShops();
                foreach ($shops as $shop) {
                    Configuration::updateValue('TARKPAY_OS_WAITING', (int) $order_state->id, false, null, (int)$shop['id_shop']);
                }
            } else {
                Configuration::updateValue('TARKPAY_OS_WAITING', (int) $order_state->id);
            }
        }

        return true;
    }
    /**
     * Delete order states
     * @return bool
     */
    public function uninstallOrderStates()
    {
        /* @var $orderState OrderState */
        $result = true;
        $collection = new PrestaShopCollection('OrderState');
        $collection->where('module_name', '=', $this->name);
        $orderStates = $collection->getResults();

        if ($orderStates == false) {
            return $result;
        }

        foreach ($orderStates as $orderState) {
            $result &= $orderState->delete();
        }

        return $result;
    }
    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitTarkpayModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitTarkpayModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Tarkpay settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->l('API key'),
                        'name' => 'TARKPAY_API_KEY',
                        'desc' => $this->l('Get required credentials from https://sandbox.tarkpay.com/ or https://tarkpay.com/'),
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Client ID'),
                        'name' => 'TARKPAY_CLIENT_ID',
                        'required' => true
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Live mode'),
                        'name' => 'TARKPAY_LIVE_MODE',
                        'is_bool' => true,
                        'required' => true,
                        'desc' => $this->l('To test in https://sandbox.tarkpay.com/ TarkPay test environment, disable live mode'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Title'),
                        'name' => 'TARKPAY_TITLE',
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'TARKPAY_LIVE_MODE' => Configuration::get('TARKPAY_LIVE_MODE', false),
            'TARKPAY_CLIENT_ID' => Configuration::get('TARKPAY_CLIENT_ID', null),
            'TARKPAY_API_KEY' => Configuration::get('TARKPAY_API_KEY', null),
            'TARKPAY_TITLE' => Configuration::get('TARKPAY_TITLE', null),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
    * Add the CSS & JavaScript files you want to be loaded in the BO.
    */
    public function hookBackOfficeHeader()
    {
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addCSS($this->_path.'/views/css/tarkpay.css');
    }

    /**
     * This hook is used to display the order confirmation page.
     */
    public function hookPaymentReturn($params)
    {
        if (!$this->active) {
            return;
        }
        if($params['order']->secure_key != Tools::getValue('key')){ # If secret keys don't match
            $this->smarty->assign(
                array(
                    'status' => 'failed',
                    'contact_url' => $this->context->link->getPageLink('contact', true),
                )
            );
            return $this->fetch('module:tarkpay/views/templates/hook/payment_return.tpl');

        }
        $orderId = Order::getOrderByCartId((int)$params['order']->id);
        $orderPaymentDatas = OrderPayment::getByOrderId($orderId);
        if(end($orderPaymentDatas)->transaction_id != Tools::getValue('payment_key')){ 
            $this->smarty->assign(
                array(
                    'status' => 'failed',
                    'contact_url' => $this->context->link->getPageLink('contact', true),
                )
            );
            return $this->fetch('module:tarkpay/views/templates/hook/payment_return.tpl');

        }
        $state = $params['order']->getCurrentState();
        $paymentStatus = $this->getTarkpayBankPaymentStatus(end($orderPaymentDatas)->transaction_id);
        if ($paymentStatus['success'] == 1)
        {
            if (in_array($state, array(Configuration::get('TARKPAY_OS_WAITING'), Configuration::get('PS_OS_PAYMENT'))) && $paymentStatus['payment'][0]['status'] == 'finished') {
                $params['order']->setCurrentState((int)Configuration::get('PS_OS_PAYMENT'));
                $totalToPaid = $params['order']->getOrdersTotalPaid() - $params['order']->getTotalPaid();
                $this->smarty->assign(array(
                    'shop_name' => $this->context->shop->name,
                    'total' => Tools::displayPrice(
                        $totalToPaid,
                        new Currency($params['order']->id_currency),
                        false
                    ),
                    'status' => 'ok',
                    'reference' => $params['order']->reference,
                    'contact_url' => $this->context->link->getPageLink('contact', true)
                ));
            } else {
                $params['order']->setCurrentState((int)Configuration::get('PS_OS_ERROR'));
                $this->smarty->assign(
                    array(
                        'status' => 'failed',
                        'contact_url' => $this->context->link->getPageLink('contact', true),
                    )
                );
            }
            return $this->fetch('module:tarkpay/views/templates/hook/payment_return.tpl');
        } else {
            /* TODO :: */ 
        }
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return [];
        }

        $this->smarty->assign(
            $this->getTemplateVarInfos()
        );

        $newOption = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $newOption->setModuleName($this->name)
            ->setCallToActionText($this->l('Tarkpay'))
            ->setAction($this->context->link->getModuleLink($this->name, 'validation', array(), true))
            ->setInputs([
                'choosedSwift' => [
                    'name' => 'choosedSwift',
                    'type' => 'hidden',
                    'value' =>null,
                ],
            ])
            ->setAdditionalInformation($this->fetch('module:tarkpay/views/templates/hook/tarkpay_intro.tpl'));
        $payment_options = [
            $newOption,
        ];
        return $payment_options;
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }
    public function getTarkpayBankIcons() {
        $url = $this->url . '/banks-icons/?client_id=' . $this->client_id;
        $content = file_get_contents($url);
        return json_decode($content, true);
    }
    public function getTarkpayBankPaymentStatus($paymentKey) {
        $opts = [
            "http" => [
                "method" => "GET",
                "header" => "Content-Type: application/json\r\n" .
                            "Authorization: Bearer " . $this->api_key . "\r\n"
            ]
        ];
        
        $context = stream_context_create($opts);
        $url = $this->url . '/payment/' . $paymentKey;
        $content = file_get_contents($url, false, $context);
        return json_decode($content, true);
    }

    public function getTemplateVarInfos()
    {
        $this->bank_icons = $this->getTarkpayBankIcons();
        $cart = $this->context->cart;
        $total = sprintf(
            $this->l('%1$s (tax incl.)'),
            Tools::displayPrice($cart->getOrderTotal(true, Cart::BOTH))
        );

        $tarkpayTitle = Configuration::get('TARKPAY_TITLE');
        if (!$tarkpayTitle) {
            $tarkpayTitle = '';
        }

        return array(
            'total' => $total,
            'tarkpayText' => $tarkpayTitle,
            'tarkpayBanks' => $this->bank_icons
        );
    }
}
